/*		Name:		serial.c
		Version:	0.0.12
		Date:		11/7/2001
		Serial interface for board or emulated board
*/
#include "flashserial.h"
#include <signal.h>
#include <unistd.h>
#include <stdlib.h>
#include <termios.h>
#include <fcntl.h>
#define PORTNAME "/dev/ttyS1"
void (* XSV_Cleanup) (void) = NULL;
int board_there;

void  serial_crash(void)         /* on errors crash out here*/
{
 g_print("Serial Crash\n");
	tcsetattr (serial_FD, TCSANOW, &serial_originalportsettings);
	close (serial_FD);
 exit(EXIT_FAILURE);
}

void serial_error(char *errorstring) /* on errors during setup crash here*/
{
 close (serial_FD);
 g_print ("Serial Error: %s\n", errorstring);
 exit(EXIT_FAILURE);
}


int serial_setup(void)      /* setup serial port settings*/
{
 struct termios serialsettings;
 	struct sigaction cleanup;

	XSV_Cleanup = serial_crash;
	memset ((void *) &serialsettings, 0, sizeof (struct termios));

	cleanup.sa_handler = (void (*)()) XSV_Cleanup;
	sigemptyset (&cleanup.sa_mask);
	cleanup.sa_flags = 0;	

 	sigaction (SIGINT, &cleanup, NULL); /* borrriinnnnggggggggg*/
	sigaction (SIGHUP, &cleanup, NULL);
	sigaction (SIGTERM, &cleanup, NULL);
	sigaction (SIGSEGV, &cleanup, NULL);
	sigaction (SIGBUS, &cleanup, NULL);

 serial_FD = open (PORTNAME, O_RDWR | O_NOCTTY| O_NDELAY);
 if (serial_FD < 0)
   serial_error("couldn't open serial port");
	fcntl (serial_FD, F_SETFL, 0);
 if (tcgetattr (serial_FD, &serial_originalportsettings) == -1)
   serial_error ("failed to serial port attributes");
 if (cfsetispeed (&serialsettings, B115200))   /* used to be B9600 */
   serial_error ("failed to set input baud rate");
 if (cfsetospeed (&serialsettings, B115200))
   serial_error ("failed to set output baud rate");

	serialsettings.c_oflag &= ~OPOST;
	serialsettings.c_iflag |= IGNPAR;
	serialsettings.c_cflag |= CS8 | CREAD | CLOCAL;
	serialsettings.c_cc[VMIN] = 0;
	serialsettings.c_cc[VTIME] = 20;   /* <-wait time*/

 if (tcsetattr (serial_FD, TCSADRAIN, &serialsettings))
   serial_error ("failed to update serial port attributes");
 return 1;
}



int board_getchar(unsigned char *to_get) /* get 1 char from board*/
{
 return board_getchararray(1,  to_get);
}

int board_sendchar(unsigned char to_send) /*send 1 char to the board*/
{
return board_sendchararray(1, &to_send);
}


int board_getb16(int* toget) /* get 16 bit int from board*/
{
 unsigned char from_board[2];
 if (2 != board_getchararray(2, from_board))     { board_there = 0; return 0;}
 *toget=from_board[0]+(from_board[1]<<8);
 return 2;
}

int board_sendb16(int tosend) /* send 16 bit int to the board*/
{
 if (1 != board_sendchar((unsigned char) tosend & 255))
                                                 { board_there = 0; return 0;}
 if (1 != board_sendchar((unsigned char) (tosend >> 8) & 255))
                                                 { board_there = 0; return 0;}
 return 2;
}
		   
int board_sendb32(int tosend) /* get 32 bit int from board*/
{
 int x=4;
 while(x--){
   if (1 != board_sendchar((unsigned char) tosend & 255))
                                                 { board_there = 0; return 0;}
   tosend = tosend>>8;
   }
 return 4;
}

int board_getb32(int* toget) /* send 32 bit int to the board*/
{
 int x=4;
 uchar temp;
 *toget=0;
 
for (x=0;x<4;x++){
   if (1 != board_getchar(&temp))            { board_there = 0; return 0;}
   *toget= *toget | temp<<(x*8);
   }
   
 return 4;
}





int board_getchararray(int char_number, unsigned char* data_ptr)
{
 int ret=char_number; /* get char array from the board*/
 int replycount=0;
 int temp;
	while (char_number){
		replycount = read(serial_FD, data_ptr, char_number);
		if (!replycount)  { g_print("timeout got %d of %d\n",ret-char_number,ret);return ret-char_number;}
		if (replycount<0) replycount=0;
		char_number -= replycount;
		data_ptr    += replycount;
//		while (replycount--) {g_print ("reply:%x\n",*data_ptr); data_ptr++;}
		}
	
 return ret;
}

int board_sendchararray(int char_number, unsigned char* data_ptr)
{
 int temp=0;
// while(temp<char_number)  g_print ("sent:%x\n",data_ptr[temp++]);
  write(serial_FD, data_ptr, char_number);
 return char_number; /* send char array to the board*/
}



